package cn.palmte.work.controller.backend;

import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import cn.palmte.work.model.DeptRepository;
import cn.palmte.work.model.Project;
import cn.palmte.work.model.ProjectBudgetIncomeDetail;
import cn.palmte.work.model.ProjectRepository;
import cn.palmte.work.model.enums.ProcessStatus;
import cn.palmte.work.model.enums.SealType;
import cn.palmte.work.service.ProjectBudgetService;
import lombok.Setter;

/**
 * @author <a href="https://github.com/TAKETODAY">Harry Yang</a>
 * @since 1.0 2022/12/8 11:03
 */
@Controller
@RequestMapping("/process")
public class ProcessController {
  static final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

  private final DeptRepository deptRepository;
  private final ProjectRepository projectRepository;
  private final ProjectBudgetService projectBudgetService;

  private final JdbcTemplate jdbcTemplate;

  public ProcessController(ProjectRepository projectRepository, DeptRepository deptRepository, ProjectBudgetService projectBudgetService, JdbcTemplate jdbcTemplate) {
    this.projectRepository = projectRepository;
    this.deptRepository = deptRepository;
    this.projectBudgetService = projectBudgetService;
    this.jdbcTemplate = jdbcTemplate;
  }

  static class FormMetadata {
    // 部门

    // 印章类型
    public Map<String, String> sealTypes;

    // 税率
    public List<Integer> taxRate;
  }

  @ResponseBody
  @GetMapping("/form/metadata")
  public FormMetadata metadata() {
    FormMetadata metadata = new FormMetadata();
    metadata.sealTypes = Arrays.stream(SealType.values())
            .collect(Collectors.toMap(SealType::getDescription, Enum::name));
    metadata.taxRate = Arrays.asList(0, 1, 3, 4, 5, 6, 9, 10, 13);
    return metadata;
  }

  /**
   * 新建流程
   */
  @GetMapping("/new")
  public String newProcess(Model model) {
    model.addAttribute("sealTypes", SealType.values());
    model.addAttribute("taxRate", Arrays.asList(0, 1, 3, 4, 5, 6, 9, 10, 13));
    return "/admin/business/process-new";
  }

  /**
   * 已办流程
   */
  @GetMapping("/completed")
  public String completed() {
    return "/admin/business/process-completed";
  }

  /**
   * 待我审核
   */
  @GetMapping("/review")
  public String review() {
    return "/admin/business/process-review";
  }

  @ResponseBody
  @GetMapping("/projects")
  public List<Map<String, Object>> query(@RequestParam String q) {
    return projectRepository.findByProjectNoOrName(q)
            .stream()
            .map(project -> {
              HashMap<String, Object> map = new HashMap<>();
              map.put("id", project.getId());
              map.put("name", project.getName());
              return map;
            })
            .collect(Collectors.toList());
  }

  public static class ProjectReturnValue {
    public final String applyDate = LocalDate.now().format(formatter);
    public String projectNo;
    public Integer projectId;

    // 申请人
    public String applyPersonName;

    // 项目类型
    public String projectType;
    // 合作类型
    public String cooperationType;

    // 合同金额
    public String contractAmount;

  }

  @ResponseBody
  @GetMapping("/projects/{id}")
  public Project query(@PathVariable int id) {
    Project byId = projectRepository.findById(id);

    // projectType
    // cooperationType
    return byId;
  }

  // 销售合同流程
  @Setter
  static class SaleContractForm {

    public Integer projectId;

    // 项目编号
    public String projectNo;

    public LocalDate applyDate;

    // 项目标题
    public String title;

    // 项目类型
    public String projectType;

    // 合作类型
    public String cooperationType;

    // 申请人
    public String applyPersonName;

    public String applyDept;

    // 申请部门领导

    public String applyDeptLeaderName;

    // 申请人电话
    public String applyPersonPhone;

    // 合同编号
    public String contractNo;

    // 合同名称
    public String contractName;

    // 客户名称
    public String clientName;

    // 用印类型
    public String[] sealType;

    // 税率
    public String taxRate;

    // 是否垫资

    // 收款条件
    public String paymentTerms;

    public ProcessStatus status;

  }

  @ResponseBody
  @PostMapping
  public void post(@RequestBody SaleContractForm form) {
    System.out.println(form);

  }

  @ResponseBody
  @GetMapping("/sale-contract-details/{id}")
  public List<ProjectBudgetIncomeDetail> queryProjectBudgetIncomeDetail(@PathVariable int id) {
    Project project = new Project();
    project.setId(id);
    return projectBudgetService.getBudgetIncomeDetail(project);
  }

  @Setter
  public static class SaleContractDetailForm {

    public int id;

    // 5 个字符
    public String expirationDate;

  }

  @ResponseBody
  @PostMapping("/sale-contract-details")
  public void saleContractDetails(@RequestBody List<SaleContractDetailForm> form) {
    jdbcTemplate.batchUpdate("update project_budget_income_detail set expiration_date =? where id =? ",
            new BatchPreparedStatementSetter() {
              @Override
              public void setValues(PreparedStatement ps, int i) throws SQLException {
                SaleContractDetailForm detailForm = form.get(i);
                ps.setString(1, detailForm.expirationDate);
                ps.setInt(2, detailForm.id);
              }

              @Override
              public int getBatchSize() {
                return form.size();
              }
            });
  }

}
